
//  File:     student.cpp
//  Author:   Chuck Stewart
//  Purpose:  Implementation of the class Student

#include <iostream>
#include <iomanip>
#include <string>
#include <vector>

#include "Student.h"
#include "stats.h"


//  Default constructor of a Student object.  Initialization of the
//  strings member variables ed in each object is automatic, since the string
//  constructor creates an empty string.  The same is true of the
//  vectors stored in the object The double variables to store the
//  averages must be initialized to 0, since this is not done
//  automatically.
Student :: Student()
  : hw_avg_(0), test_avg_(0), final_avg_(0)
{ }


//  Construct a student from the name and id.
Student :: Student( std::string const& first, std::string const& last,
                    std::string const& id )
  : first_name_(first), last_name_(last), id_number_(id),
    hw_avg_(0), test_avg_(0), final_avg_(0)
{}


void
Student :: reset()
{
  first_name_.clear();
  last_name_.clear();
  id_number_.clear();
  hw_scores_.clear();
  hw_avg_ = 0.0;
  test_scores_.clear();
  test_avg_ = 0.0;
  final_avg_ = 0.0;
}


void 
Student :: add_name_and_id( std::string const& first, std::string const& last,
                            std::string const& id )
{
  first_name_ = first;
  last_name_ = last;
  id_number_ = id;
}


void
Student :: add_hw( int hw_score )
{
  hw_scores_.push_back( hw_score );
}
  

void 
Student :: add_test( int test_score )
{
  test_scores_.push_back( test_score );
}



//  Compute and fill in the hw, test and final average for the
//  student.  This uses the avg_and_stddev function that we wrote for
//  statistics calculations, but retains only the average.

void 
Student :: compute_averages( double hw_weight )
{
  double dummy_stddev;

  if ( !hw_scores_.empty() )
    avg_and_stddev( hw_scores_, hw_avg_, dummy_stddev );

  if ( !test_scores_.empty() )
    avg_and_stddev( test_scores_, test_avg_, dummy_stddev );

  final_avg_ = hw_weight * hw_avg_ + (1 - hw_weight) * test_avg_;
}


std::ostream&
Student :: output_name( std::ostream& out_str ) const
{
  out_str << last_name_ << ", " << first_name_;
  return out_str;
}


std::ostream&
Student :: output_averages( std::ostream& out_str ) const
{
  out_str << std::setprecision(3) << hw_avg_ << " "
          << test_avg_ << " " << final_avg_ << std::endl;
  return out_str;
}
          

//  Return true if name of stu1 should come before stu2 in
//  alphabetical order.  (Strictly speaking, what is provided is based
//  on lexicographic order, but if all names start with capital
//  letters, this will be the same.)
bool less_names( const Student& stu1,
                 const Student& stu2 )
{
  return
    stu1.last_name() < stu2.last_name() ||
    ( stu1.last_name() == stu2.last_name() &&
      stu1.first_name() < stu2.first_name() );
}

