
//  File:     student.h
//  Author:   Chuck Stewart
//  Purpose:  Header for declaration of student record class and
//    associated functions.

#ifndef student_h_
#define student_h_

#include <iostream>
#include <string>
#include <vector>

class Student {
public:
  //  Default constructor
  Student();

  //  Constructor given a name and an id
  Student( std::string const& first, std::string const& last,
	   std::string const& id );

  //  Functions for manipulating the information stored in the Student
  //  object
  void reset();     // empty and reset all local variables
  void add_name_and_id( std::string const& first, std::string const& last,
			std::string const& id );
  void add_hw( int hw_score );
  void add_test( int test_score );

  //  Main computation function.
  void compute_averages( double hw_weight );
  
  //  Access the results
  const std::string& first_name() const { return first_name_; }
  const std::string& last_name() const { return last_name_; }
  const std::string& id_number() const { return id_number_; }

  double hw_avg() const { return hw_avg_; }
  double test_avg() const { return test_avg_; }

  //  Output the final information.
  std::ostream& output_name( std::ostream& out_str ) const;
  std::ostream& output_averages( std::ostream& out_str ) const;

private:
  std::string first_name_;
  std::string last_name_;
  std::string id_number_;
  std::vector<int> hw_scores_;
  double hw_avg_;
  std::vector<int> test_scores_;
  double test_avg_;
  double final_avg_;
};

bool less_names( const Student& stu1, const Student& stu2 );

#endif

